var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

// external imports
import React from 'react';
import Text from 'react-native-web/dist/components/Text';
import StyleSheet from 'react-native-web/dist/apis/StyleSheet';
import Animated from 'react-native-web/dist/apis/Animated';
import Easing from 'react-native-web/dist/apis/Easing';
import TouchableWithoutFeedback from 'react-native-web/dist/components/Touchable/TouchableWithoutFeedback';

import PropTypes from 'prop-types';
// local imports
import { baseDim } from '../../styles';
import { styles, containerSizes, textSizes, sizeConstraints } from './styles';

class BaseButton extends React.Component {
    _pressIn(...args) {
        // if there is a press handler to deal with
        if (this.props.onPressIn) {
            // call it
            this.props.onPressIn(...arg);
        }
        this.setState({
            animation: Animated.timing(this.state.opacity, {
                toValue: 1,
                duration: 100
            })
        }, () => {
            // start the animation
            this.state.animation.start();
        });
    }

    _pressOut(...arg) {
        // stop the keypress animation if its running
        // if there is a press handler to deal with
        if (this.props.onPressOut) {
            // call it
            this.props.onPressOut(...arg);
        }

        // start the animation to return to normal state
        Animated.timing(this.state.opacity, {
            toValue: 0,
            duration: 150
        }).start();
    }

    constructor(...args) {
        super(...args);

        // initial state
        this.state = {
            opacity: new Animated.Value(0)

            // function binds
        };this._pressIn = this._pressIn.bind(this);
        this._pressOut = this._pressOut.bind(this);
    }

    render() {
        const _props = this.props,
              {
            style,
            children,
            size,
            activeColor,
            constrainSize,
            defaultColor,
            textColor,
            onPress
        } = _props,
              unused = _objectWithoutProperties(_props, ['style', 'children', 'size', 'activeColor', 'constrainSize', 'defaultColor', 'textColor', 'onPress']);

        return React.createElement(
            TouchableWithoutFeedback,
            {
                onPressIn: this._pressIn,
                onPressOut: this._pressOut,
                onPress: onPress
            },
            React.createElement(
                Animated.View,
                _extends({}, unused, {
                    style: [styles.container, containerSizes[size], constrainSize && sizeConstraints[size], {
                        backgroundColor: this.state.opacity.interpolate({
                            inputRange: [0, 1],
                            outputRange: [defaultColor, activeColor]
                        })
                    }, style]
                }),
                children
            )
        );
    }
}

export const buttonDefaultProps = {
    size: 'medium',
    constrainSize: true
};

export const buttonPropTypes = {
    size: PropTypes.string,
    constrainSize: PropTypes.bool
};

BaseButton.defaultProps = _extends({}, buttonDefaultProps);

BaseButton.propTypes = _extends({}, buttonPropTypes, {
    defaultColor: PropTypes.string,
    activeColor: PropTypes.string,
    hoverColor: PropTypes.string,
    textColor: PropTypes.string
});

export default BaseButton;